"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FfmpegProcess = void 0;
/* Copyright(C) 2017-2020, HJD (https://github.com/hjdhjd). All rights reserved.
 *
 * doorbird-ffmpeg.ts: FFmpeg capability validation and process control.
 *
 * This module is heavily inspired by the homebridge and homebridge-camera-ffmpeg source code and
 * borrows heavily from both. Thank you for your contributions to the HomeKit world.
 */
const child_process_1 = require("child_process");
const dgram_1 = require("dgram");
const execa_1 = __importDefault(require("execa"));
class FfmpegProcess {
    constructor(delegate, sessionId, command, returnPort, callback) {
        this.killing = false;
        this.debug = delegate.platform.debug.bind(this);
        this.delegate = delegate;
        this.log = delegate.platform.log;
        let started = false;
        this.debug("%s: ffmpeg command: %s %s", delegate.name, delegate.videoProcessor, command);
        const socket = dgram_1.createSocket("udp4");
        // Handle network errors.
        socket.on("error", (error) => {
            this.log.error("%s: Socket error: ", delegate.name, error.name);
            delegate.stopStream(sessionId);
        });
        // Kill zombie video streams.
        socket.on("message", () => {
            if (this.timeout) {
                clearTimeout(this.timeout);
            }
            this.timeout = setTimeout(() => {
                delegate.platform.log.info("%s: Device appears to be inactive for over 5 seconds. Stopping stream.", delegate.name);
                delegate.controller.forceStopStreamingSession(sessionId);
                delegate.stopStream(sessionId);
            }, 5000);
        });
        socket.bind(returnPort);
        // Execute the command line.
        this.process = child_process_1.spawn(delegate.videoProcessor, command.split(/\s+/), { env: process.env });
        if (this.process.stdin) {
            this.process.stdin.on("error", (error) => {
                if (!error.message.includes("EPIPE")) {
                    this.log.error("%s: FFmpeg error: %s", delegate.name, error.message);
                }
            });
        }
        if (this.process.stderr) {
            this.process.stderr.on("data", (data) => {
                if (!started) {
                    started = true;
                    this.debug("%s: Received the first frame.", delegate.name);
                    // Always remember to execute the callback once we're setup.
                    if (callback) {
                        callback();
                    }
                }
                // Debugging and additional logging, if requested.
                if (delegate.platform.debugMode) {
                    data.toString().split(/\n/).forEach((line) => {
                        this.log(line);
                    });
                }
            });
        }
        // Error handling.
        this.process.on("error", (error) => {
            this.log.error("%s: Unable to start stream: %s.", delegate.name, error.message);
            if (callback) {
                callback(new Error("ffmpeg process creation failed!"));
                delegate.stopStream(sessionId);
            }
        });
        // Handle the end of our process - graceful and otherwise.
        this.process.on("exit", (code, signal) => {
            const message = "%s: ffmpeg exited with code: " + code + " and signal: " + signal;
            if (code === null || code === 255) {
                if (this.killing) {
                    this.debug(message + " (Expected)", delegate.name);
                }
                else {
                    this.log.error(message + " (Unexpected)", delegate.name);
                }
            }
            else {
                this.log.error(message + " (Error)", delegate.name);
                delegate.stopStream(sessionId);
                if (!started && callback) {
                    callback(new Error(message));
                }
                else {
                    delegate.controller.forceStopStreamingSession(sessionId);
                }
            }
        });
    }
    // Cleanup.
    stop() {
        this.killing = true;
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
        this.process.kill("SIGKILL");
    }
    // Validate whether or not we have a specific codec available to us in FFmpeg.
    static async codecEnabled(videoProcessor, codec) {
        const output = await execa_1.default(videoProcessor, ["-codecs"]);
        return output.stdout.includes(codec);
    }
}
exports.FfmpegProcess = FfmpegProcess;
//# sourceMappingURL=doorbird-ffmpeg.js.map